// Parser.h: interface for the CParser class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_PARSER_H__00015480_6EB3_48D2_861B_ABF94842826A__INCLUDED_)
#define AFX_PARSER_H__00015480_6EB3_48D2_861B_ABF94842826A__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//
// The BNF for the XML tags are as followings:
// 
// element		:	:=    EmptyElemTag 
//				    | STag content ETag 
//
//		STag		::=    '<' Name (S Attribute)* S? '>'
//Attribute			::=    Name Eq AttValue 
//
// EmptyElemTag		::=    '<' Name (S Attribute)* S? '/> 
//  ETag			::=    '</' Name S? '>' 

//Comment    ::=    '<!--' ((Char - '-') | ('-' (Char - '-')))* '-->' 
//
//  PI    ::=    '<?' PITarget (S (Char* - (Char* '?>' Char*)))? '?>' 
//  PITarget    ::=    Name - (('X' | 'x') ('M' | 'm') ('L' | 'l')) 
// content ::= (element | PCData | Reference | CDSect | PI | Comment

 
class CElement;

class CBaseParser
{
public:
	explicit CBaseParser(CElement*parent):m_pParent(parent),m_bValid(true){}
	virtual bool Parse( CScanner* pScanner)=0;
	inline bool IsValid() const{return m_bValid;}
	inline CElement* GetParent() const{return m_pParent;}
	inline void SetParent(CElement* pParent){m_pParent=pParent;}
	inline void SetValid(bool b=true){m_bValid=b;}


protected:
	CElement* m_pParent;
 	bool	m_bValid;

};
//
// class CCommentParser
//
// The class that parse the XML comments tags.
//
class CCommentParser:public CBaseParser
{
public:
	explicit CCommentParser(CElement*Parent):CBaseParser(Parent){}
	virtual bool Parse( CScanner* pScanner);

private:
	CCommentParser();

	
};

//
// class CPIParser
// 
// parse the "Process Instruction" part of the XML elements
//
class CPIParser:public CBaseParser
{
public:
	explicit CPIParser(CElement*Parent):CBaseParser(Parent){}
	virtual bool Parse( CScanner* pScanner);
};

//
// class CChardata
//
// Parse the Chardata in between the STag and ETag
class CChardata:public CBaseParser
{
public:
	explicit CChardata(CElement*Parent):CBaseParser(Parent){}
	CChardata(CChardata const&obj):
				m_StringValue(obj.m_StringValue),CBaseParser(obj.m_pParent){}
				
	CChardata& operator=(CChardata const&obj)
	{
		if(this!=&obj) m_StringValue=obj.m_StringValue;
		return *this;
	}
	virtual bool Parse( CScanner* pScanner);
	inline operator bool()  const{return m_StringValue.m_pBegin!=NULL&&m_StringValue.m_pEnd!=NULL;}
	

	CStringValue & GetStringObj() const 
	{return const_cast<CStringValue &>(m_StringValue);}

	void SetStringObj(CStringValue& s){ m_StringValue=s;}  


	friend std::ostream& operator<<(std::ostream&os, CChardata const& obj)
	{
		if (obj)	os<<obj.GetStringObj();
		return os;
	}

private:
	CStringValue m_StringValue;

};

//
// class CAttribute
//
// Parse the Chardata in between the STag and ETag
class CAttribute:public CBaseParser
{
public:
	explicit CAttribute(CElement*Parent):CBaseParser(Parent){}
	CAttribute(CStringValue const&Name,CStringValue const&Value,CElement*Parent):
										CBaseParser(Parent),
										m_AttributeName(Name),
										m_AttributeValue(Value){}

	virtual bool Parse( CScanner* pScanner);

	const CStringValue& GetAttrNameObj() const {return m_AttributeName;}
	const CStringValue& GetAttrValueObj() const {return m_AttributeValue;}

	void SetValue(const char* Value);


 
	friend std::ostream& operator<<(std::ostream& os, CAttribute const&obj)
	{
		return os<<obj.GetAttrNameObj()<<"=\""<<obj.GetAttrValueObj()<<"\"";
	}

 	CAttribute(CAttribute  const &obj):	
				CBaseParser(obj.m_pParent),
				m_AttributeName(obj.m_AttributeName),
				m_AttributeValue(obj.m_AttributeValue)
	{}

	
private:
	CAttribute();
	CStringValue m_AttributeName;
	CStringValue m_AttributeValue;
};


//
// class CStagParser
//
// Parse the information inside start tag
class CStagParser:public CBaseParser
{
public:
	explicit CStagParser(CElement*Parent):
							CBaseParser(Parent),
							m_bIsEmptyElementTag(false),
							m_bIsComment(false),
							m_bIsPI(false){}
	virtual bool Parse( CScanner* pScanner);
 
	inline const CStringValue& GetNameObj() const {return m_StringValue;}

	inline bool IsEmptyElementTag() const {return m_bIsEmptyElementTag;}
	inline bool IsCommentTag() const {return m_bIsComment;}
	inline bool IsPITag() const {return m_bIsPI;}

private:
   CStagParser();
	CStringValue m_StringValue;
	bool     m_bIsEmptyElementTag;
	bool     m_bIsComment;
	bool     m_bIsPI;
};

//
// class CEtagParser
//
// Parse the end of tag information 
// </' Name S? '>'

class CEtagParser:public CBaseParser
{
public:
	CEtagParser(CElement*p):CBaseParser(p){}
	virtual bool Parse( CScanner* pScanner);
 	inline const CStringValue& GetNameObj() const {return m_StringValue;}

private:
	CEtagParser();
	CStringValue m_StringValue;
};


//
// class CContent
//
// Desc:
// The Content parser is parsing the string whose format is
// content ::= (element | PCData | Reference | CDSect | PI | Comment
//
class CContent:public CBaseParser
{
public:
	explicit CContent(CElement*Parent):CBaseParser(Parent){}
	virtual bool Parse( CScanner* pScanner);

private:
	CContent();

};




//
// class CElement
//
// The class whose instance is going to parse the entire element.
//
class CElementIterator;
class CElement:public CBaseParser
{
public:

	explicit CElement(CElement*parent):
						CBaseParser(parent),
						m_CharData(parent),
						m_pPrevSibling(NULL),
						m_pNextSibling(NULL),
						m_pFirstChild(NULL)
						{}

	static CElement* CreateNewElement(const char* Name, CElement*parent=NULL);

 
	~CElement(){Clean();}
	void Delete(){delete this;}

	//
	// Operations
	//
	virtual bool Parse( CScanner* pScanner);

	//
	// Modifiers
	//

	//
	// For element Name
	//
	void SetName(CStringValue& s){m_StringValue=s;}
	
	//
	// For element attributes
	//
	void AddAttribute(CAttribute*p){m_AttributeList.push_back(p);}
	void AddAttributePair(const char*Name, const char*Value);
	bool ModifyAttribute(const char*Name, const char*Value);
	bool ModifyAttributeByIndex(int index, const char*Value);
	bool DeleteAttributeByName(const char*Name);
	bool DeleteAttributeByIndex(int index);

	//
	// For element data
	//
	void SetCharData(CChardata& cd){m_CharData=cd;}
	void SetCharData(CStringValue& s);

	//
	// For the children elements
	//
	void AddChildElement(CElement*p);
	bool DeleteChild (CElementIterator& it);
	bool DeleteChild (CElement*p);

	//
	// Accessors
	//
	CElementIterator GetFirstChild();
 	inline CStringValue const& GetName() const {return m_StringValue;}
	bool GetAttribute(const char*name, vector<char>&ret);
	int  GetAttributeCount();
	bool GetAttributePairByIndex(int index, vector<char>&attName, vector<char>&attValue);
	bool GetCharData(vector<char>&ret);

	//
	// Stream operations
	//


	friend std::ostream& operator<< (std::ostream& ,CElement const&);
	friend std::ostream& operator<< (std::ostream& ,CElement &);
	friend class CElementIterator;

private:
	CElement();
	void Clean();
	CStringValue m_StringValue;
	std::vector <CAttribute*>   m_AttributeList;
	CChardata                   m_CharData;

	//
	// Define the structure for sub elements that chains all the siblings
	//
	CElement* m_pPrevSibling;
	CElement* m_pNextSibling;
	CElement* m_pFirstChild; 
};


//
// Class CElementIterator
//
// This is a friend class of the class CElement
//
class CElementIterator
{
public:
	//
	// The constructor that accept the CElement parameter.
	//
	explicit CElementIterator (CElement* obj);

	CElementIterator& operator++ ();
	inline bool IsValid() const{return m_bValid&&m_pCurrentElement!=NULL;}
	inline void Invalidate()  {m_bValid=false;m_pCurrentElement=NULL;}

	CElementIterator(CElementIterator const &);
	CElementIterator& operator= (CElementIterator const&);

	CElement* operator*(){return GetElementPtr();}
	CElement* GetElementPtr() {return m_bValid? m_pCurrentElement:NULL;}

	void Delete();


 private:
//
// Hide the default constructor
//
	CElementIterator();
 	bool	m_bValid;

	CElement* m_pCurrentElement;
};
#endif // !defined(AFX_PARSER_H__00015480_6EB3_48D2_861B_ABF94842826A__INCLUDED_)
